@@ -1,7 +1,7 @@
 /*
  * gedit-changecase-plugin.c
  * 
- * Copyright (C) 2004-2005 - Paolo Borelli
+ * Copyright (C) 2004-2010 - Paolo Borelli
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id$
  */
 
 #ifdef HAVE_CONFIG_H
@@ -26,15 +25,13 @@
 
 #include "gedit-changecase-plugin.h"
 
-#include <glib/gi18n-lib.h>
+#include <glib/gi18n.h>
 #include <gmodule.h>
 
+#include <gedit/gedit-window.h>
+#include <gedit/gedit-window-activatable.h>
 #include <gedit/gedit-debug.h>
 
-#define WINDOW_DATA_KEY "GeditChangecasePluginWindowData"
-
-GEDIT_PLUGIN_REGISTER_TYPE(GeditChangecasePlugin, gedit_changecase_plugin)
-
 typedef enum {
 	TO_UPPER_CASE,
 	TO_LOWER_CASE,
@@ -42,6 +39,29 @@ typedef enum {
 	TO_TITLE_CASE,
 } ChangeCaseChoice;
 
+struct _GeditChangecasePluginPrivate
+{
+	GeditWindow    *window;
+
+	GtkActionGroup *action_group;
+	guint           ui_id;
+};
+
+enum
+{
+	PROP_0,
+	PROP_WINDOW
+};
+
+static void gedit_window_activatable_iface_init (GeditWindowActivatableInterface *iface);
+
+G_DEFINE_DYNAMIC_TYPE_EXTENDED (GeditChangecasePlugin,
+				gedit_changecase_plugin,
+				PEAS_TYPE_EXTENSION_BASE,
+				0,
+				G_IMPLEMENT_INTERFACE_DYNAMIC (GEDIT_TYPE_WINDOW_ACTIVATABLE,
+							       gedit_window_activatable_iface_init))
+
 static void
 do_upper_case (GtkTextBuffer *buffer,
                GtkTextIter   *start,
@@ -254,6 +274,32 @@ static void
 gedit_changecase_plugin_init (GeditChangecasePlugin *plugin)
 {
 	gedit_debug_message (DEBUG_PLUGINS, "GeditChangecasePlugin initializing");
+
+	plugin->priv = G_TYPE_INSTANCE_GET_PRIVATE (plugin,
+						    GEDIT_TYPE_CHANGECASE_PLUGIN,
+						    GeditChangecasePluginPrivate);
+}
+
+static void
+gedit_changecase_plugin_dispose (GObject *object)
+{
+	GeditChangecasePlugin *plugin = GEDIT_CHANGECASE_PLUGIN (object);
+
+	gedit_debug_message (DEBUG_PLUGINS, "GeditChangecasePlugin disponsing");
+
+	if (plugin->priv->window != NULL)
+	{
+		g_object_unref (plugin->priv->window);
+		plugin->priv->window = NULL;
+	}
+
+	if (plugin->priv->action_group != NULL)
+	{
+		g_object_unref (plugin->priv->action_group);
+		plugin->priv->action_group = NULL;
+	}
+
+	G_OBJECT_CLASS (gedit_changecase_plugin_parent_class)->dispose (object);
 }
 
 static void
@@ -264,23 +310,48 @@ gedit_changecase_plugin_finalize (GObject *object)
 	gedit_debug_message (DEBUG_PLUGINS, "GeditChangecasePlugin finalizing");
 }
 
-typedef struct
-{
-	GtkActionGroup *action_group;
-	guint           ui_id;
-} WindowData;
-
 static void
-free_window_data (WindowData *data)
+gedit_changecase_plugin_set_property (GObject      *object,
+                                      guint         prop_id,
+                                      const GValue *value,
+                                      GParamSpec   *pspec)
 {
-	g_return_if_fail (data != NULL);
+	GeditChangecasePlugin *plugin = GEDIT_CHANGECASE_PLUGIN (object);
 
-	g_slice_free (WindowData, data);
+	switch (prop_id)
+	{
+		case PROP_WINDOW:
+			plugin->priv->window = GEDIT_WINDOW (g_value_dup_object (value));
+			break;
+
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+			break;
+	}
 }
 
 static void
-update_ui_real (GeditWindow  *window,
-		WindowData   *data)
+gedit_changecase_plugin_get_property (GObject    *object,
+                                      guint       prop_id,
+                                      GValue     *value,
+                                      GParamSpec *pspec)
+{
+	GeditChangecasePlugin *plugin = GEDIT_CHANGECASE_PLUGIN (object);
+
+	switch (prop_id)
+	{
+		case PROP_WINDOW:
+			g_value_set_object (value, plugin->priv->window);
+			break;
+
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+			break;
+	}
+}
+
+static void
+update_ui (GeditChangecasePlugin *plugin)
 {
 	GtkTextView *view;
 	GtkAction *action;
@@ -288,7 +359,7 @@ update_ui_real (GeditWindow  *window,
 
 	gedit_debug (DEBUG_PLUGINS);
 
-	view = GTK_TEXT_VIEW (gedit_window_get_active_view (window));
+	view = GTK_TEXT_VIEW (gedit_window_get_active_view (plugin->priv->window));
 
 	if (view != NULL)
 	{
@@ -299,98 +370,108 @@ update_ui_real (GeditWindow  *window,
 			     gtk_text_buffer_get_has_selection (buffer));
 	}
 
-	action = gtk_action_group_get_action (data->action_group,
+	action = gtk_action_group_get_action (plugin->priv->action_group,
 					      "ChangeCase");
 	gtk_action_set_sensitive (action, sensitive);
 }
 
 static void
-impl_activate (GeditPlugin *plugin,
-	       GeditWindow *window)
+gedit_changecase_plugin_activate (GeditWindowActivatable *activatable)
 {
+	GeditChangecasePluginPrivate *priv;
 	GtkUIManager *manager;
-	WindowData *data;
 	GError *error = NULL;
 
 	gedit_debug (DEBUG_PLUGINS);
 
-	data = g_slice_new (WindowData);
+	priv = GEDIT_CHANGECASE_PLUGIN (activatable)->priv;
 
-	manager = gedit_window_get_ui_manager (window);
+	manager = gedit_window_get_ui_manager (priv->window);
 
-	data->action_group = gtk_action_group_new ("GeditChangecasePluginActions");
-	gtk_action_group_set_translation_domain (data->action_group, 
+	priv->action_group = gtk_action_group_new ("GeditChangecasePluginActions");
+	gtk_action_group_set_translation_domain (priv->action_group, 
 						 GETTEXT_PACKAGE);
-	gtk_action_group_add_actions (data->action_group,
+	gtk_action_group_add_actions (priv->action_group,
 				      action_entries,
-				      G_N_ELEMENTS (action_entries), 
-				      window);
+				      G_N_ELEMENTS (action_entries),
+				      priv->window);
 
-	gtk_ui_manager_insert_action_group (manager, data->action_group, -1);
+	gtk_ui_manager_insert_action_group (manager, priv->action_group, -1);
 
-	data->ui_id = gtk_ui_manager_add_ui_from_string (manager,
+	priv->ui_id = gtk_ui_manager_add_ui_from_string (manager,
 							 submenu,
 							 -1,
 							 &error);
-	if (data->ui_id == 0)
+	if (priv->ui_id == 0)
 	{
 		g_warning ("%s", error->message);
-		free_window_data (data);
 		return;
 	}
 
-	g_object_set_data_full (G_OBJECT (window), 
-				WINDOW_DATA_KEY, 
-				data,
-				(GDestroyNotify) free_window_data);
-
-	update_ui_real (window, data);
+	update_ui (GEDIT_CHANGECASE_PLUGIN (activatable));
 }
 
 static void
-impl_deactivate	(GeditPlugin *plugin,
-		 GeditWindow *window)
+gedit_changecase_plugin_deactivate (GeditWindowActivatable *activatable)
 {
+	GeditChangecasePluginPrivate *priv;
 	GtkUIManager *manager;
-	WindowData *data;
 
 	gedit_debug (DEBUG_PLUGINS);
 
-	manager = gedit_window_get_ui_manager (window);
+	priv = GEDIT_CHANGECASE_PLUGIN (activatable)->priv;
 
-	data = (WindowData *) g_object_get_data (G_OBJECT (window), WINDOW_DATA_KEY);
-	g_return_if_fail (data != NULL);
+	manager = gedit_window_get_ui_manager (priv->window);
 
-	gtk_ui_manager_remove_ui (manager, data->ui_id);
-	gtk_ui_manager_remove_action_group (manager, data->action_group);
-
-	g_object_set_data (G_OBJECT (window), WINDOW_DATA_KEY, NULL);	
+	gtk_ui_manager_remove_ui (manager, priv->ui_id);
+	gtk_ui_manager_remove_action_group (manager, priv->action_group);
 }
 
 static void
-impl_update_ui (GeditPlugin *plugin,
-		GeditWindow *window)
+gedit_changecase_plugin_update_state (GeditWindowActivatable *activatable)
 {
-	WindowData *data;
-
 	gedit_debug (DEBUG_PLUGINS);
 
-	data = (WindowData *) g_object_get_data (G_OBJECT (window), WINDOW_DATA_KEY);
-	g_return_if_fail (data != NULL);
-
-	update_ui_real (window, data);
+	update_ui (GEDIT_CHANGECASE_PLUGIN (activatable));
 }
 
 static void
 gedit_changecase_plugin_class_init (GeditChangecasePluginClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	GeditPluginClass *plugin_class = GEDIT_PLUGIN_CLASS (klass);
 
+	object_class->dispose = gedit_changecase_plugin_dispose;
 	object_class->finalize = gedit_changecase_plugin_finalize;
+	object_class->set_property = gedit_changecase_plugin_set_property;
+	object_class->get_property = gedit_changecase_plugin_get_property;
 
-	plugin_class->activate = impl_activate;
-	plugin_class->deactivate = impl_deactivate;
-	plugin_class->update_ui = impl_update_ui;
+	g_object_class_override_property (object_class, PROP_WINDOW, "window");
+
+	g_type_class_add_private (klass, sizeof (GeditChangecasePluginPrivate));
 }
-/* ex:ts=8:noet: */
+
+static void
+gedit_window_activatable_iface_init (GeditWindowActivatableInterface *iface)
+{
+	iface->activate = gedit_changecase_plugin_activate;
+	iface->deactivate = gedit_changecase_plugin_deactivate;
+	iface->update_state = gedit_changecase_plugin_update_state;
+}
+
+static void
+gedit_changecase_plugin_class_finalize (GeditChangecasePluginClass *klass)
+{
+}
+
+
+G_MODULE_EXPORT void
+peas_register_types (PeasObjectModule *module)
+{
+	gedit_changecase_plugin_register_type (G_TYPE_MODULE (module));
+
+	peas_object_module_register_extension_type (module,
+						    GEDIT_TYPE_WINDOW_ACTIVATABLE,
+						    GEDIT_TYPE_CHANGECASE_PLUGIN);
+}
+
+/* ex:set ts=8 noet: */
