@@ -1,3 +1,23 @@
+/*
+ * gedit-file-browser-messages.c
+ *
+ * Copyright (C) 2006 - Jesse van den Kieboom <jesse@icecrew.nl>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
+
 #include "gedit-file-browser-messages.h"
 #include "gedit-file-browser-store.h"
 #include <gedit/gedit-message.h>
@@ -9,7 +29,7 @@
 
 typedef struct
 {
-	GeditWindow *window;	
+	GeditWindow  *window;
 	GeditMessage *message;
 } MessageCacheData;
 
@@ -23,19 +43,19 @@ typedef struct
 
 	GList *merge_ids;
 	GtkActionGroup *merged_actions;
-	
+
 	GeditMessageBus *bus;
 	GeditFileBrowserWidget *widget;
 	GHashTable *row_tracking;
-	
+
 	GHashTable *filters;
 } WindowData;
 
 typedef struct
 {
 	gulong id;
-	
-	GeditWindow *window;
+
+	GeditWindow  *window;
 	GeditMessage *message;
 } FilterData;
 
@@ -46,10 +66,10 @@ window_data_new (GeditWindow            *window,
 	WindowData *data = g_slice_new (WindowData);
 	GtkUIManager *manager;
 	GList *groups;
-	
+
 	data->bus = gedit_window_get_message_bus (window);
 	data->widget = widget;
-	data->row_tracking = g_hash_table_new_full (g_str_hash, 
+	data->row_tracking = g_hash_table_new_full (g_str_hash,
 						    g_str_equal,
 						    (GDestroyNotify)g_free,
 						    (GDestroyNotify)gtk_tree_row_reference_free);
@@ -58,22 +78,22 @@ window_data_new (GeditWindow            *window,
 					       g_str_equal,
 					       (GDestroyNotify)g_free,
 					       NULL);
-	
+
 	manager = gedit_file_browser_widget_get_ui_manager (widget);
 
 	data->merge_ids = NULL;
 	data->merged_actions = gtk_action_group_new ("MessageMergedActions");
-	
+
 	groups = gtk_ui_manager_get_action_groups (manager);
 	gtk_ui_manager_insert_action_group (manager, data->merged_actions, g_list_length (groups));
-	
+
 	g_object_set_data (G_OBJECT (window), WINDOW_DATA_KEY, data);
 
 	return data;
 }
 
 static WindowData *
-get_window_data (GeditWindow * window)
+get_window_data (GeditWindow *window)
 {
 	return (WindowData *) (g_object_get_data (G_OBJECT (window), WINDOW_DATA_KEY));
 }
@@ -84,13 +104,13 @@ window_data_free (GeditWindow *window)
 	WindowData *data = get_window_data (window);
 	GtkUIManager *manager;
 	GList *item;
-		
-	g_hash_table_destroy (data->row_tracking);	
+
+	g_hash_table_destroy (data->row_tracking);
 	g_hash_table_destroy (data->filters);
 
 	manager = gedit_file_browser_widget_get_ui_manager (data->widget);
 	gtk_ui_manager_remove_action_group (manager, data->merged_actions);
-	
+
 	for (item = data->merge_ids; item; item = item->next)
 		gtk_ui_manager_remove_ui (manager, GPOINTER_TO_INT (item->data));
 
@@ -103,19 +123,19 @@ window_data_free (GeditWindow *window)
 }
 
 static FilterData *
-filter_data_new (GeditWindow  *window,  
+filter_data_new (GeditWindow  *window,
 		 GeditMessage *message)
 {
 	FilterData *data = g_slice_new (FilterData);
 	WindowData *wdata;
-	
+
 	data->window = window;
 	data->id = 0;
 	data->message = message;
-	
+
 	wdata = get_window_data (window);
-	
-	g_hash_table_insert (wdata->filters, 
+
+	g_hash_table_insert (wdata->filters,
 			     gedit_message_type_identifier (gedit_message_get_object_path (message),
 			                                    gedit_message_get_method (message)),
 			     data);
@@ -128,10 +148,10 @@ filter_data_free (FilterData *data)
 {
 	WindowData *wdata = get_window_data (data->window);
 	gchar *identifier;
-	
+
 	identifier = gedit_message_type_identifier (gedit_message_get_object_path (data->message),
 			                            gedit_message_get_method (data->message));
-			                            
+
 	g_hash_table_remove (wdata->filters, identifier);
 	g_free (identifier);
 
@@ -140,16 +160,16 @@ filter_data_free (FilterData *data)
 }
 
 static GtkTreePath *
-track_row_lookup (WindowData  *data, 
+track_row_lookup (WindowData  *data,
 		  const gchar *id)
 {
 	GtkTreeRowReference *ref;
-	
+
 	ref = (GtkTreeRowReference *)g_hash_table_lookup (data->row_tracking, id);
-	
+
 	if (!ref)
 		return NULL;
-	
+
 	return gtk_tree_row_reference_get_path (ref);
 }
 
@@ -161,14 +181,14 @@ message_cache_data_free (MessageCacheData *data)
 }
 
 static MessageCacheData *
-message_cache_data_new (GeditWindow            *window,
-			GeditMessage           *message)
+message_cache_data_new (GeditWindow  *window,
+			GeditMessage *message)
 {
 	MessageCacheData *data = g_slice_new (MessageCacheData);
-	
+
 	data->window = window;
 	data->message = message;
-	
+
 	return data;
 }
 
@@ -178,13 +198,16 @@ message_get_root_cb (GeditMessageBus *bus,
 		     WindowData      *data)
 {
 	GeditFileBrowserStore *store;
-	gchar *uri;
-	
+	GFile *location;
+
 	store = gedit_file_browser_widget_get_browser_store (data->widget);
-	uri = gedit_file_browser_store_get_virtual_root (store);
-	
-	gedit_message_set (message, "uri", uri, NULL);
-	g_free (uri);
+	location = gedit_file_browser_store_get_virtual_root (store);
+
+	if (location)
+	{
+		gedit_message_set (message, "location", location, NULL);
+		g_object_unref (location);
+	}
 }
 
 static void
@@ -192,14 +215,14 @@ message_set_root_cb (GeditMessageBus *bus,
 		     GeditMessage    *message,
 		     WindowData      *data)
 {
-	gchar *root = NULL;
-	gchar *virtual = NULL;
-	
-	gedit_message_get (message, "uri", &root, NULL);
-	
+	GFile *root;
+	GFile *virtual = NULL;
+
+	gedit_message_get (message, "location", &root, NULL);
+
 	if (!root)
 		return;
-	
+
 	if (gedit_message_has_key (message, "virtual"))
 		gedit_message_get (message, "virtual", &virtual, NULL);
 
@@ -207,9 +230,6 @@ message_set_root_cb (GeditMessageBus *bus,
 		gedit_file_browser_widget_set_root_and_virtual_root (data->widget, root, virtual);
 	else
 		gedit_file_browser_widget_set_root (data->widget, root, TRUE);
-	
-	g_free (root);
-	g_free (virtual);
 }
 
 static void
@@ -221,128 +241,132 @@ message_set_emblem_cb (GeditMessageBus *bus,
 	gchar *emblem = NULL;
 	GtkTreePath *path;
 	GeditFileBrowserStore *store;
-	
+
 	gedit_message_get (message, "id", &id, "emblem", &emblem, NULL);
-	
+
 	if (!id || !emblem)
 	{
 		g_free (id);
 		g_free (emblem);
-		
+
 		return;
 	}
-	
+
 	path = track_row_lookup (data, id);
-	
+
 	if (path != NULL)
 	{
 		GError *error = NULL;
 		GdkPixbuf *pixbuf;
-		
-		pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), 
-						   emblem, 
-						   10, 
-						   0, 
+
+		pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+						   emblem,
+						   10,
+						   0,
 						   &error);
-		
+
 		if (pixbuf)
 		{
 			GValue value = { 0, };
 			GtkTreeIter iter;
-			
+
 			store = gedit_file_browser_widget_get_browser_store (data->widget);
-			
+
 			if (gtk_tree_model_get_iter (GTK_TREE_MODEL (store), &iter, path))
 			{
 				g_value_init (&value, GDK_TYPE_PIXBUF);
 				g_value_set_object (&value, pixbuf);
-			
-				gedit_file_browser_store_set_value (store, 
+
+				gedit_file_browser_store_set_value (store,
 								    &iter,
 								    GEDIT_FILE_BROWSER_STORE_COLUMN_EMBLEM,
 								    &value);
-			
+
 				g_value_unset (&value);
 			}
-			
+
 			g_object_unref (pixbuf);
 		}
-		
+
 		if (error)
 			g_error_free (error);
 	}
-	
+
 	g_free (id);
 	g_free (emblem);
 }
 
 static gchar *
 item_id (const gchar *path,
-	 const gchar *uri)
+	 GFile *location)
 {
-	return g_strconcat (path, "::", uri, NULL);
+	gchar *uri;
+	gchar *id;
+
+	uri = g_file_get_uri (location);
+	id = g_strconcat (path, "::", uri, NULL);
+	g_free (uri);
+
+	return id;
 }
 
 static gchar *
 track_row (WindowData            *data,
 	   GeditFileBrowserStore *store,
 	   GtkTreePath           *path,
-	   const gchar		 *uri)
+	   GFile		 *location)
 {
 	GtkTreeRowReference *ref;
 	gchar *id;
 	gchar *pathstr;
-	
+
 	pathstr = gtk_tree_path_to_string (path);
-	id = item_id (pathstr, uri);
-	
+	id = item_id (pathstr, location);
+
 	ref = gtk_tree_row_reference_new (GTK_TREE_MODEL (store), path);
 	g_hash_table_insert (data->row_tracking, g_strdup (id), ref);
-	
+
 	g_free (pathstr);
-	
+
 	return id;
 }
 
 static void
-set_item_message (WindowData   *data, 
+set_item_message (WindowData   *data,
 		  GtkTreeIter  *iter,
 		  GtkTreePath  *path,
 		  GeditMessage *message)
 {
 	GeditFileBrowserStore *store;
-	gchar *uri = NULL;
+	GFile *location;
 	guint flags = 0;
 	gchar *track_id;
-	
+
 	store = gedit_file_browser_widget_get_browser_store (data->widget);
-	
+
 	gtk_tree_model_get (GTK_TREE_MODEL (store), iter,
-			    GEDIT_FILE_BROWSER_STORE_COLUMN_URI, &uri,
+			    GEDIT_FILE_BROWSER_STORE_COLUMN_LOCATION, &location,
 			    GEDIT_FILE_BROWSER_STORE_COLUMN_FLAGS, &flags,
 			    -1);
-	
-	if (!uri)
+
+	if (!location)
 		return;
 
 	if (path && gtk_tree_path_get_depth (path) != 0)
-		track_id = track_row (data, store, path, uri);
+		track_id = track_row (data, store, path, location);
 	else
 		track_id = NULL;
 
 	gedit_message_set (message,
 			   "id", track_id,
-			   "uri", uri,
+			   "location", location,
 			   NULL);
-	
+
 	if (gedit_message_has_key (message, "is_directory"))
-	{
-		gedit_message_set (message, 
+		gedit_message_set (message,
 				   "is_directory", FILE_IS_DIR (flags),
 				   NULL);
-	}			   
 
-	g_free (uri);
 	g_free (track_id);
 }
 
@@ -353,31 +377,28 @@ custom_message_filter_func (GeditFileBrowserWidget *widget,
 			    FilterData             *data)
 {
 	WindowData *wdata = get_window_data (data->window);
-	gchar *uri = NULL;
+	GFile *location;
 	guint flags = 0;
 	gboolean filter = FALSE;
 	GtkTreePath *path;
-	
-	gtk_tree_model_get (GTK_TREE_MODEL (store), iter, 
-			    GEDIT_FILE_BROWSER_STORE_COLUMN_URI, &uri,
+
+	gtk_tree_model_get (GTK_TREE_MODEL (store), iter,
+			    GEDIT_FILE_BROWSER_STORE_COLUMN_LOCATION, &location,
 			    GEDIT_FILE_BROWSER_STORE_COLUMN_FLAGS, &flags,
 			    -1);
-	
-	if (!uri || FILE_IS_DUMMY (flags))
-	{
-		g_free (uri);
+
+	if (!location || FILE_IS_DUMMY (flags))
 		return FALSE;
-	}
-	
+
 	path = gtk_tree_model_get_path (GTK_TREE_MODEL (store), iter);
 	set_item_message (wdata, iter, path, data->message);
 	gtk_tree_path_free (path);
-	
+
 	gedit_message_set (data->message, "filter", filter, NULL);
 
 	gedit_message_bus_send_message_sync (wdata->bus, data->message);
 	gedit_message_get (data->message, "filter", &filter, NULL);
-	
+
 	return !filter;
 }
 
@@ -393,50 +414,48 @@ message_add_filter_cb (GeditMessageBus *bus,
 	GeditMessage *cbmessage;
 	FilterData *filter_data;
 	WindowData *data = get_window_data (window);
-	
-	gedit_message_get (message, 
+
+	gedit_message_get (message,
 			   "object_path", &object_path,
 			   "method", &method,
 			   NULL);
-	
-	// Check if there exists such a 'callback' message
+
+	/* Check if there exists such a 'callback' message */
 	if (!object_path || !method)
 	{
 		g_free (object_path);
 		g_free (method);
-		
+
 		return;
 	}
-	
+
 	message_type = gedit_message_bus_lookup (bus, object_path, method);
-	
+
 	if (!message_type)
 	{
 		g_free (object_path);
 		g_free (method);
-		
+
 		return;
 	}
-	
-	// Check if the message type has the correct arguments
+
+	/* Check if the message type has the correct arguments */
 	if (gedit_message_type_lookup (message_type, "id") != G_TYPE_STRING ||
-	    gedit_message_type_lookup (message_type, "uri") != G_TYPE_STRING ||
+	    gedit_message_type_lookup (message_type, "location") != G_TYPE_FILE ||
 	    gedit_message_type_lookup (message_type, "is_directory") != G_TYPE_BOOLEAN ||
 	    gedit_message_type_lookup (message_type, "filter") != G_TYPE_BOOLEAN)
-	{
 		return;
-	}
-	
+
 	cbmessage = gedit_message_type_instantiate (message_type,
 						    "id", NULL,
-						    "uri", NULL,
+						    "location", NULL,
 						    "is_directory", FALSE,
 						    "filter", FALSE,
 						    NULL);
 
-	// Register the custom filter on the widget
+	/* Register the custom filter on the widget */
 	filter_data = filter_data_new (window, cbmessage);
-	id = gedit_file_browser_widget_add_filter (data->widget, 
+	id = gedit_file_browser_widget_add_filter (data->widget,
 						   (GeditFileBrowserWidgetFilterFunc)custom_message_filter_func,
 						   filter_data,
 						   (GDestroyNotify)filter_data_free);
@@ -450,12 +469,12 @@ message_remove_filter_cb (GeditMessageBus *bus,
 		          WindowData      *data)
 {
 	gulong id = 0;
-	
+
 	gedit_message_get (message, "id", &id, NULL);
-	
+
 	if (!id)
 		return;
-	
+
 	gedit_file_browser_widget_remove_filter (data->widget, id);
 }
 
@@ -465,7 +484,7 @@ message_up_cb (GeditMessageBus *bus,
 	       WindowData      *data)
 {
 	GeditFileBrowserStore *store = gedit_file_browser_widget_get_browser_store (data->widget);
-	
+
 	gedit_file_browser_store_set_virtual_root_up (store);
 }
 
@@ -501,12 +520,12 @@ message_set_show_hidden_cb (GeditMessageBus *bus,
 	gboolean active = FALSE;
 	GeditFileBrowserStore *store;
 	GeditFileBrowserStoreFilterMode mode;
-	
+
 	gedit_message_get (message, "active", &active, NULL);
-	
+
 	store = gedit_file_browser_widget_get_browser_store (data->widget);
 	mode = gedit_file_browser_store_get_filter_mode (store);
-	
+
 	if (active)
 		mode &= ~GEDIT_FILE_BROWSER_STORE_FILTER_MODE_HIDE_HIDDEN;
 	else
@@ -523,12 +542,12 @@ message_set_show_binary_cb (GeditMessageBus *bus,
 	gboolean active = FALSE;
 	GeditFileBrowserStore *store;
 	GeditFileBrowserStoreFilterMode mode;
-	
+
 	gedit_message_get (message, "active", &active, NULL);
-	
+
 	store = gedit_file_browser_widget_get_browser_store (data->widget);
 	mode = gedit_file_browser_store_get_filter_mode (store);
-	
+
 	if (active)
 		mode &= ~GEDIT_FILE_BROWSER_STORE_FILTER_MODE_HIDE_BINARY;
 	else
@@ -563,12 +582,12 @@ message_add_context_item_cb (GeditMessageBus *bus,
 	gchar *name;
 	GtkUIManager *manager;
 	guint merge_id;
-	
-	gedit_message_get (message, 
-			   "action", &action, 
-			   "path", &path, 
+
+	gedit_message_get (message,
+			   "action", &action,
+			   "path", &path,
 			   NULL);
-	
+
 	if (!action || !path)
 	{
 		if (action)
@@ -577,20 +596,20 @@ message_add_context_item_cb (GeditMessageBus *bus,
 		g_free (path);
 		return;
 	}
-	
+
 	gtk_action_group_add_action (data->merged_actions, action);
 	manager = gedit_file_browser_widget_get_ui_manager (data->widget);
 	name = g_strconcat (gtk_action_get_name (action), "MenuItem", NULL);
 	merge_id = gtk_ui_manager_new_merge_id (manager);
-	
-	gtk_ui_manager_add_ui (manager, 
+
+	gtk_ui_manager_add_ui (manager,
 			       merge_id,
 			       path,
 			       name,
 			       gtk_action_get_name (action),
 			       GTK_UI_MANAGER_AUTO,
 			       FALSE);
-	
+
 	if (gtk_ui_manager_get_widget (manager, path))
 	{
 		data->merge_ids = g_list_prepend (data->merge_ids, GINT_TO_POINTER (merge_id));
@@ -600,7 +619,7 @@ message_add_context_item_cb (GeditMessageBus *bus,
 	{
 		gedit_message_set (message, "id", 0, NULL);
 	}
-	
+
 	g_object_unref (action);
 	g_free (path);
 	g_free (name);
@@ -613,14 +632,14 @@ message_remove_context_item_cb (GeditMessageBus *bus,
 {
 	guint merge_id = 0;
 	GtkUIManager *manager;
-	
+
 	gedit_message_get (message, "id", &merge_id, NULL);
-	
+
 	if (merge_id == 0)
 		return;
-	
+
 	manager = gedit_file_browser_widget_get_ui_manager (data->widget);
-		
+
 	data->merge_ids = g_list_remove (data->merge_ids, GINT_TO_POINTER (merge_id));
 	gtk_ui_manager_remove_ui (manager, merge_id);
 }
@@ -644,26 +663,26 @@ register_methods (GeditWindow            *window,
 	WindowData *data = get_window_data (window);
 
 	/* Register method calls */
-	gedit_message_bus_register (bus, 
-				    MESSAGE_OBJECT_PATH, "get_root", 
+	gedit_message_bus_register (bus,
+				    MESSAGE_OBJECT_PATH, "get_root",
 				    1,
-				    "uri", G_TYPE_STRING,
+				    "location", G_TYPE_FILE,
 				    NULL);
- 
-	gedit_message_bus_register (bus, 
-				    MESSAGE_OBJECT_PATH, "set_root", 
-				    1, 
-				    "uri", G_TYPE_STRING,
+
+	gedit_message_bus_register (bus,
+				    MESSAGE_OBJECT_PATH, "set_root",
+				    1,
+				    "location", G_TYPE_FILE,
 				    "virtual", G_TYPE_STRING,
 				    NULL);
-				    
+
 	gedit_message_bus_register (bus,
 				    MESSAGE_OBJECT_PATH, "set_emblem",
 				    0,
 				    "id", G_TYPE_STRING,
 				    "emblem", G_TYPE_STRING,
 				    NULL);
-	
+
 	gedit_message_bus_register (bus,
 				    MESSAGE_OBJECT_PATH, "add_filter",
 				    1,
@@ -671,13 +690,13 @@ register_methods (GeditWindow            *window,
 				    "method", G_TYPE_STRING,
 				    "id", G_TYPE_ULONG,
 				    NULL);
-	
+
 	gedit_message_bus_register (bus,
 				    MESSAGE_OBJECT_PATH, "remove_filter",
 				    0,
 				    "id", G_TYPE_ULONG,
 				    NULL);
-	
+
 	gedit_message_bus_register (bus,
 				    MESSAGE_OBJECT_PATH, "add_context_item",
 				    1,
@@ -691,22 +710,22 @@ register_methods (GeditWindow            *window,
 				    0,
 				    "id", G_TYPE_UINT,
 				    NULL);
-	
+
 	gedit_message_bus_register (bus, MESSAGE_OBJECT_PATH, "up", 0, NULL);
-	
+
 	gedit_message_bus_register (bus, MESSAGE_OBJECT_PATH, "history_back", 0, NULL);
 	gedit_message_bus_register (bus, MESSAGE_OBJECT_PATH, "history_forward", 0, NULL);
-	
+
 	gedit_message_bus_register (bus, MESSAGE_OBJECT_PATH, "refresh", 0, NULL);
 
-	gedit_message_bus_register (bus, 
-				    MESSAGE_OBJECT_PATH, "set_show_hidden", 
-				    0, 
+	gedit_message_bus_register (bus,
+				    MESSAGE_OBJECT_PATH, "set_show_hidden",
+				    0,
 				    "active", G_TYPE_BOOLEAN,
 				    NULL);
-	gedit_message_bus_register (bus, 
+	gedit_message_bus_register (bus,
 				    MESSAGE_OBJECT_PATH, "set_show_binary",
-				    0, 
+				    0,
 				    "active", G_TYPE_BOOLEAN,
 				    NULL);
 
@@ -733,10 +752,10 @@ register_methods (GeditWindow            *window,
 	BUS_CONNECT (bus, history_forward, data);
 
 	BUS_CONNECT (bus, refresh, data);
-	
+
 	BUS_CONNECT (bus, set_show_hidden, data);
 	BUS_CONNECT (bus, set_show_binary, data);
-	
+
 	BUS_CONNECT (bus, show_bookmarks, data);
 	BUS_CONNECT (bus, show_files, data);
 
@@ -749,23 +768,19 @@ store_row_inserted (GeditFileBrowserStore *store,
 		    GtkTreeIter           *iter,
 		    MessageCacheData      *data)
 {
-	gchar *uri = NULL;
 	guint flags = 0;
 
-	gtk_tree_model_get (GTK_TREE_MODEL (store), iter, 
-			    GEDIT_FILE_BROWSER_STORE_COLUMN_URI, &uri,
+	gtk_tree_model_get (GTK_TREE_MODEL (store), iter,
 			    GEDIT_FILE_BROWSER_STORE_COLUMN_FLAGS, &flags,
 			    -1);
-	
+
 	if (!FILE_IS_DUMMY (flags) && !FILE_IS_FILTERED (flags))
 	{
 		WindowData *wdata = get_window_data (data->window);
-		
+
 		set_item_message (wdata, iter, path, data->message);
 		gedit_message_bus_send_message_sync (wdata->bus, data->message);
 	}
-	
-	g_free (uri);
 }
 
 static void
@@ -774,26 +789,22 @@ store_row_deleted (GeditFileBrowserStore *store,
 		   MessageCacheData      *data)
 {
 	GtkTreeIter iter;
-	gchar *uri = NULL;
 	guint flags = 0;
-	
+
 	if (!gtk_tree_model_get_iter (GTK_TREE_MODEL (store), &iter, path))
 		return;
-	
-	gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, 
-			    GEDIT_FILE_BROWSER_STORE_COLUMN_URI, &uri,
+
+	gtk_tree_model_get (GTK_TREE_MODEL (store), &iter,
 			    GEDIT_FILE_BROWSER_STORE_COLUMN_FLAGS, &flags,
 			    -1);
-	
+
 	if (!FILE_IS_DUMMY (flags) && !FILE_IS_FILTERED (flags))
 	{
 		WindowData *wdata = get_window_data (data->window);
-		
+
 		set_item_message (wdata, &iter, path, data->message);
 		gedit_message_bus_send_message_sync (wdata->bus, data->message);
 	}
-	
-	g_free (uri);
 }
 
 static void
@@ -802,20 +813,20 @@ store_virtual_root_changed (GeditFileBrowserStore *store,
 			    MessageCacheData      *data)
 {
 	WindowData *wdata = get_window_data (data->window);
-	gchar *uri;
-	
-	uri = gedit_file_browser_store_get_virtual_root (store);
-	
-	if (!uri)
+	GFile *vroot;
+
+	vroot = gedit_file_browser_store_get_virtual_root (store);
+
+	if (!vroot)
 		return;
-	
+
 	gedit_message_set (data->message,
-			   "uri", uri,
+			   "location", vroot,
 			   NULL);
-			   
+
 	gedit_message_bus_send_message_sync (wdata->bus, data->message);
-	
-	g_free (uri);
+
+	g_object_unref (vroot);
 }
 
 static void
@@ -825,11 +836,11 @@ store_begin_loading (GeditFileBrowserStore *store,
 {
 	GtkTreePath *path;
 	WindowData *wdata = get_window_data (data->window);
-	
+
 	path = gtk_tree_model_get_path (GTK_TREE_MODEL (store), iter);
-	
+
 	set_item_message (wdata, iter, path, data->message);
-	
+
 	gedit_message_bus_send_message_sync (wdata->bus, data->message);
 	gtk_tree_path_free (path);
 }
@@ -841,15 +852,15 @@ store_end_loading (GeditFileBrowserStore *store,
 {
 	GtkTreePath *path;
 	WindowData *wdata = get_window_data (data->window);
-	
+
 	path = gtk_tree_model_get_path (GTK_TREE_MODEL (store), iter);
-	
+
 	set_item_message (wdata, iter, path, data->message);
-	
+
 	gedit_message_bus_send_message_sync (wdata->bus, data->message);
 	gtk_tree_path_free (path);
 }
-		    
+
 static void
 register_signals (GeditWindow            *window,
 		  GeditFileBrowserWidget *widget)
@@ -861,7 +872,7 @@ register_signals (GeditWindow            *window,
 	GeditMessageType *begin_loading_type;
 	GeditMessageType *end_loading_type;
 	GeditMessageType *root_changed_type;
-	
+
 	GeditMessage *message;
 	WindowData *data;
 
@@ -870,28 +881,28 @@ register_signals (GeditWindow            *window,
 				    MESSAGE_OBJECT_PATH, "root_changed",
 				    0,
 				    "id", G_TYPE_STRING,
-				    "uri", G_TYPE_STRING,
+				    "location", G_TYPE_FILE,
 				    NULL);
-	
+
 	begin_loading_type = gedit_message_bus_register (bus,
 				    MESSAGE_OBJECT_PATH, "begin_loading",
 				    0,
 				    "id", G_TYPE_STRING,
-				    "uri", G_TYPE_STRING,
+				    "location", G_TYPE_FILE,
 				    NULL);
 
 	end_loading_type = gedit_message_bus_register (bus,
 				    MESSAGE_OBJECT_PATH, "end_loading",
 				    0,
 				    "id", G_TYPE_STRING,
-				    "uri", G_TYPE_STRING,
+				    "location", G_TYPE_FILE,
 				    NULL);
 
 	inserted_type = gedit_message_bus_register (bus,
 						    MESSAGE_OBJECT_PATH, "inserted",
 						    0,
 						    "id", G_TYPE_STRING,
-						    "uri", G_TYPE_STRING,
+						    "location", G_TYPE_FILE,
 						    "is_directory", G_TYPE_BOOLEAN,
 						    NULL);
 
@@ -899,46 +910,46 @@ register_signals (GeditWindow            *window,
 						   MESSAGE_OBJECT_PATH, "deleted",
 						   0,
 						   "id", G_TYPE_STRING,
-						   "uri", G_TYPE_STRING,
+						   "location", G_TYPE_FILE,
 						   "is_directory", G_TYPE_BOOLEAN,
 						   NULL);
 
 	store = gedit_file_browser_widget_get_browser_store (widget);
-	
-	message = gedit_message_type_instantiate (inserted_type, 
+
+	message = gedit_message_type_instantiate (inserted_type,
 						  "id", NULL,
-						  "uri", NULL, 
-						  "is_directory", FALSE, 
+						  "location", NULL,
+						  "is_directory", FALSE,
 						  NULL);
 
 	data = get_window_data (window);
 
-	data->row_inserted_id = 
-		g_signal_connect_data (store, 
-				       "row-inserted", 
-				       G_CALLBACK (store_row_inserted), 
+	data->row_inserted_id =
+		g_signal_connect_data (store,
+				       "row-inserted",
+				       G_CALLBACK (store_row_inserted),
 				       message_cache_data_new (window, message),
 				       (GClosureNotify)message_cache_data_free,
 				       0);
 
-	message = gedit_message_type_instantiate (deleted_type, 
-						  "id", NULL, 
-						  "uri", NULL,
-						  "is_directory", FALSE, 
+	message = gedit_message_type_instantiate (deleted_type,
+						  "id", NULL,
+						  "location", NULL,
+						  "is_directory", FALSE,
 						  NULL);
-	data->row_deleted_id = 
-		g_signal_connect_data (store, 
-				       "row-deleted", 
-				       G_CALLBACK (store_row_deleted), 
+	data->row_deleted_id =
+		g_signal_connect_data (store,
+				       "row-deleted",
+				       G_CALLBACK (store_row_deleted),
 				       message_cache_data_new (window, message),
 				       (GClosureNotify)message_cache_data_free,
 				       0);
-	
+
 	message = gedit_message_type_instantiate (root_changed_type,
 						  "id", NULL,
-						  "uri", NULL,
+						  "location", NULL,
 						  NULL);
-	data->root_changed_id = 
+	data->root_changed_id =
 		g_signal_connect_data (store,
 				       "notify::virtual-root",
 				       G_CALLBACK (store_virtual_root_changed),
@@ -948,9 +959,9 @@ register_signals (GeditWindow            *window,
 
 	message = gedit_message_type_instantiate (begin_loading_type,
 						  "id", NULL,
-						  "uri", NULL,
-						  NULL);	
-	data->begin_loading_id = 
+						  "location", NULL,
+						  NULL);
+	data->begin_loading_id =
 		g_signal_connect_data (store,
 				      "begin_loading",
 				       G_CALLBACK (store_begin_loading),
@@ -960,9 +971,9 @@ register_signals (GeditWindow            *window,
 
 	message = gedit_message_type_instantiate (end_loading_type,
 						  "id", NULL,
-						  "uri", NULL,
+						  "location", NULL,
 						  NULL);
-	data->end_loading_id = 
+	data->end_loading_id =
 		g_signal_connect_data (store,
 				       "end_loading",
 				       G_CALLBACK (store_end_loading),
@@ -980,28 +991,28 @@ message_unregistered (GeditMessageBus  *bus,
 							   gedit_message_type_get_method (message_type));
 	FilterData *data;
 	WindowData *wdata = get_window_data (window);
-	
+
 	data = g_hash_table_lookup (wdata->filters, identifier);
-	
+
 	if (data)
 		gedit_file_browser_widget_remove_filter (wdata->widget, data->id);
-	
+
 	g_free (identifier);
 }
 
-void 
-gedit_file_browser_messages_register (GeditWindow            *window, 
+void
+gedit_file_browser_messages_register (GeditWindow            *window,
 				      GeditFileBrowserWidget *widget)
 {
 	window_data_new (window, widget);
-	
+
 	register_methods (window, widget);
 	register_signals (window, widget);
-	
+
 	g_signal_connect (gedit_window_get_message_bus (window),
 			  "unregistered",
 			  G_CALLBACK (message_unregistered),
-			  window);	
+			  window);
 }
 
 static void
@@ -1009,15 +1020,15 @@ cleanup_signals (GeditWindow *window)
 {
 	WindowData *data = get_window_data (window);
 	GeditFileBrowserStore *store;
-	
+
 	store = gedit_file_browser_widget_get_browser_store (data->widget);
-	
+
 	g_signal_handler_disconnect (store, data->row_inserted_id);
 	g_signal_handler_disconnect (store, data->row_deleted_id);
 	g_signal_handler_disconnect (store, data->root_changed_id);
 	g_signal_handler_disconnect (store, data->begin_loading_id);
 	g_signal_handler_disconnect (store, data->end_loading_id);
-	
+
 	g_signal_handlers_disconnect_by_func (data->bus, message_unregistered, window);
 }
 
@@ -1025,9 +1036,10 @@ void
 gedit_file_browser_messages_unregister (GeditWindow *window)
 {
 	GeditMessageBus *bus = gedit_window_get_message_bus (window);
-		
+
 	cleanup_signals (window);
 	gedit_message_bus_unregister_all (bus, MESSAGE_OBJECT_PATH);
 
 	window_data_free (window);
 }
+/* ex:ts=8:noet: */
